import 'dart:async';
import 'dart:io';
import 'dart:ui';

import 'package:equatable/equatable.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import '../../model/video_folder_item.dart';
import '../../model/video_item.dart';
import '../../repository/file_repository.dart';
import '../../repository/video_repository.dart';
import '../../util/common_util.dart';

part 'video_folders_event.dart';

part 'video_folders_state.dart';

class VideoFoldersBloc extends Bloc<VideoFoldersEvent, VideoFoldersState> {
  final VideoRepository _videoRepository;
  final FileRepository _fileRepository;

  VideoFoldersBloc(
      {required VideoRepository videoRepository,
      required FileRepository fileRepository})
      : _videoRepository = videoRepository,
        _fileRepository = fileRepository,
        super(VideoFoldersState()) {
    on<VideoFoldersSubscriptionRequested>(_onSubscriptionRequested);
    on<VideoFoldersOpenStatusChanged>(_onOpenFolderStatusChanged);
    on<VideoFoldersCheckedChanged>(_onVideoFoldersCheckedChanged);
    on<VideoFoldersKeyStatusChanged>(_onKeyStatusChanged);
    on<VideoFoldersCheckAll>(_onCheckAll);
    on<VideoFoldersClearAll>(_onClearAll);
    on<VideoFoldersVideosCheckedChanged>(_onVideosCheckedChanged);
    on<VideoFoldersMenuStatusChanged>(_onMenuStatusChanged);
    on<VideoFoldersDeleteSubmitted>(_onDeleteFoldersSubmitted);
    on<VideoFoldersCopySubmitted>(_onCopySubmitted);
    on<VideoFoldersCopyStatusChanged>(_onCopyStatusChanged);
    on<VideoFoldersCancelCopy>(_onCancelCopy);
    on<VideoFoldersVideosCopySubmitted>(_onCopyVideosSubmitted);
    on<VideoFoldersDeleteVideosSubmitted>(_onDeleteVideosSubmitted);
    on<VideoFoldersUploadVideos>(_onUploadVideos);
    on<VideoFoldersUploadStatusChanged>(_onUploadStatusChanged);
    on<VideoFoldersDownloadVideoFoldersToLocal>(_onDownloadVideoFoldersToLocal);
    on<VideoFoldersDownloadVideosToLocal>(_onDownloadVideosToLocal);
  }

  void _onSubscriptionRequested(VideoFoldersSubscriptionRequested event,
      Emitter<VideoFoldersState> emit) async {
    emit(state.copyWith(status: VideoFoldersStatus.loading));

    try {
      List<VideoFolderItem> videoFolders =
          await _videoRepository.getAllVideoFolders();
      emit(state.copyWith(
          status: VideoFoldersStatus.success, videoFolders: videoFolders));
    } on Exception catch (e) {
      emit(state.copyWith(
          status: VideoFoldersStatus.failure,
          failureReason: CommonUtil.convertHttpError(e)));
    }
  }

  void _onOpenFolderStatusChanged(VideoFoldersOpenStatusChanged event,
      Emitter<VideoFoldersState> emit) async {
    emit(state.copyWith(videoFolderOpenStatus: event.status));

    if (event.status.isOpened) {
      emit(state.copyWith(
          loadVideosInFolderStatus: state.loadVideosInFolderStatus
              .copyWith(status: VideoFoldersStatus.loading)));

      try {
        List<VideoItem> videos =
            await _videoRepository.getVideosInFolder(event.status.current!.id);
        emit(state.copyWith(
            loadVideosInFolderStatus: state.loadVideosInFolderStatus
                .copyWith(status: VideoFoldersStatus.success, videos: videos)));
      } on Exception catch (e) {
        emit(state.copyWith(
            loadVideosInFolderStatus: state.loadVideosInFolderStatus.copyWith(
                status: VideoFoldersStatus.failure,
                error: CommonUtil.convertHttpError(e))));
      }
    } else {
      emit(state.copyWith(
          loadVideosInFolderStatus:
              state.loadVideosInFolderStatus.copyWith(checkedVideos: [])));
    }
  }

  void _onVideoFoldersCheckedChanged(
      VideoFoldersCheckedChanged event, Emitter<VideoFoldersState> emit) {
    List<VideoFolderItem> videoFolders = state.videoFolders;
    List<VideoFolderItem> checkedVideoFolders = [...state.checkedVideoFolders];
    VideoFolderItem folder = event.videoFolder;

    VideoFoldersBoardKeyStatus keyStatus = state.keyStatus;

    if (!checkedVideoFolders.contains(folder)) {
      if (keyStatus == VideoFoldersBoardKeyStatus.ctrlDown) {
        checkedVideoFolders.add(folder);
      } else if (keyStatus == VideoFoldersBoardKeyStatus.shiftDown) {
        if (checkedVideoFolders.length == 0) {
          checkedVideoFolders.add(folder);
        } else if (checkedVideoFolders.length == 1) {
          int index = checkedVideoFolders.indexOf(checkedVideoFolders[0]);

          int current = videoFolders.indexOf(folder);

          if (current > index) {
            checkedVideoFolders = videoFolders.sublist(index, current + 1);
          } else {
            checkedVideoFolders = videoFolders.sublist(index, current + 1);
          }
        } else {
          int maxIndex = 0;
          int minIndex = 0;

          for (int i = 0; i < checkedVideoFolders.length; i++) {
            VideoFolderItem current = checkedVideoFolders[i];
            int index = videoFolders.indexOf(current);
            if (index < 0) {
              continue;
            }

            if (index > maxIndex) {
              maxIndex = index;
            }

            if (index < minIndex) {
              minIndex = index;
            }
          }

          int current = videoFolders.indexOf(folder);

          if (current >= minIndex && current <= maxIndex) {
            checkedVideoFolders = videoFolders.sublist(current, maxIndex + 1);
          } else if (current < minIndex) {
            checkedVideoFolders = videoFolders.sublist(current, maxIndex + 1);
          } else if (current > maxIndex) {
            checkedVideoFolders = videoFolders.sublist(minIndex, current + 1);
          }
        }
      } else {
        checkedVideoFolders.clear();
        checkedVideoFolders.add(folder);
      }
    } else {
      if (keyStatus == VideoFoldersBoardKeyStatus.ctrlDown) {
        checkedVideoFolders.remove(folder);
      } else if (keyStatus == VideoFoldersBoardKeyStatus.shiftDown) {
        checkedVideoFolders.remove(folder);
      }
    }

    emit(state.copyWith(checkedVideoFolders: checkedVideoFolders));
  }

  void _onKeyStatusChanged(
      VideoFoldersKeyStatusChanged event, Emitter<VideoFoldersState> emit) {
    emit(state.copyWith(keyStatus: event.keyStatus));
  }

  void _onCheckAll(
      VideoFoldersCheckAll event, Emitter<VideoFoldersState> emit) {
    bool isFolderOpened = state.videoFolderOpenStatus.isOpened;
    if (isFolderOpened) {
      emit(state.copyWith(
          loadVideosInFolderStatus: state.loadVideosInFolderStatus.copyWith(
              checkedVideos: [...state.loadVideosInFolderStatus.videos])));
    } else {
      emit(state.copyWith(checkedVideoFolders: [...state.videoFolders]));
    }
  }

  void _onClearAll(
      VideoFoldersClearAll event, Emitter<VideoFoldersState> emit) {
    bool isFolderOpened = state.videoFolderOpenStatus.isOpened;
    if (isFolderOpened) {
      emit(state.copyWith(
          loadVideosInFolderStatus:
              state.loadVideosInFolderStatus.copyWith(checkedVideos: [])));
    } else {
      emit(state.copyWith(checkedVideoFolders: []));
    }
  }

  void _onVideosCheckedChanged(
      VideoFoldersVideosCheckedChanged event, Emitter<VideoFoldersState> emit) {
    List<VideoItem> videos = state.loadVideosInFolderStatus.videos;
    List<VideoItem> checkedVideos = [
      ...state.loadVideosInFolderStatus.checkedVideos
    ];
    VideoItem video = event.video;

    VideoFoldersBoardKeyStatus keyStatus = state.keyStatus;

    if (!checkedVideos.contains(video)) {
      if (keyStatus == VideoFoldersBoardKeyStatus.ctrlDown) {
        checkedVideos.add(video);
      } else if (keyStatus == VideoFoldersBoardKeyStatus.shiftDown) {
        if (checkedVideos.length == 0) {
          checkedVideos.add(video);
        } else if (checkedVideos.length == 1) {
          int index = checkedVideos.indexOf(checkedVideos[0]);

          int current = videos.indexOf(video);

          if (current > index) {
            checkedVideos = videos.sublist(index, current + 1);
          } else {
            checkedVideos = videos.sublist(index, current + 1);
          }
        } else {
          int maxIndex = 0;
          int minIndex = 0;

          for (int i = 0; i < checkedVideos.length; i++) {
            VideoItem current = checkedVideos[i];
            int index = videos.indexOf(current);
            if (index < 0) {
              continue;
            }

            if (index > maxIndex) {
              maxIndex = index;
            }

            if (index < minIndex) {
              minIndex = index;
            }
          }

          int current = videos.indexOf(video);

          if (current >= minIndex && current <= maxIndex) {
            checkedVideos = videos.sublist(current, maxIndex + 1);
          } else if (current < minIndex) {
            checkedVideos = videos.sublist(current, maxIndex + 1);
          } else if (current > maxIndex) {
            checkedVideos = videos.sublist(minIndex, current + 1);
          }
        }
      } else {
        checkedVideos.clear();
        checkedVideos.add(video);
      }
    } else {
      if (keyStatus == VideoFoldersBoardKeyStatus.ctrlDown) {
        checkedVideos.remove(video);
      } else if (keyStatus == VideoFoldersBoardKeyStatus.shiftDown) {
        checkedVideos.remove(video);
      }
    }

    emit(state.copyWith(
        loadVideosInFolderStatus: state.loadVideosInFolderStatus
            .copyWith(checkedVideos: checkedVideos)));
  }

  void _onMenuStatusChanged(
      VideoFoldersMenuStatusChanged event, Emitter<VideoFoldersState> emit) {
    emit(state.copyWith(openMenuStatus: event.status));
  }

  void _onDeleteFoldersSubmitted(VideoFoldersDeleteSubmitted event,
      Emitter<VideoFoldersState> emit) async {
    emit(state.copyWith(deleteStatus: VideoFoldersDeleteStatus.loading));

    try {
      await _videoRepository.deleteVideoFolders(event.videoFolders);

      List<VideoFolderItem> videoFolders = [...state.videoFolders];
      List<VideoFolderItem> checkedVideoFolders = [
        ...state.checkedVideoFolders
      ];

      videoFolders.removeWhere((folder) => event.videoFolders.contains(folder));
      checkedVideoFolders
          .removeWhere((folder) => event.videoFolders.contains(folder));

      emit(state.copyWith(
          deleteStatus: VideoFoldersDeleteStatus.success,
          videoFolders: videoFolders,
          checkedVideoFolders: checkedVideoFolders));
    } on Exception catch (e) {
      emit(state.copyWith(
          deleteStatus: VideoFoldersDeleteStatus.failure,
          failureReason: CommonUtil.convertHttpError(e)));
    }
  }

  void _onCopySubmitted(
      VideoFoldersCopySubmitted event, Emitter<VideoFoldersState> emit) {
    emit(state.copyWith(
        copyStatus: VideoFoldersCopyStatusUnit(
            fileType: VideoFoldersFileType.folder,
            status: VideoFoldersCopyStatus.start)));

    String? fileName = null;

    if (event.folders.length == 1) {
      fileName = "${event.folders.single.name}.zip";
    }

    _videoRepository.copyVideoFoldersTo(
        fileName: fileName,
        videoFolders: event.folders,
        dir: event.dir,
        onProgress: (fileName, current, total) {
          add(VideoFoldersCopyStatusChanged(VideoFoldersCopyStatusUnit(
              fileType: VideoFoldersFileType.folder,
              status: VideoFoldersCopyStatus.copying,
              fileName: fileName,
              current: current,
              total: total)));
        },
        onDone: (fileName) {
          add(VideoFoldersCopyStatusChanged(VideoFoldersCopyStatusUnit(
              fileType: VideoFoldersFileType.folder,
              status: VideoFoldersCopyStatus.success,
              fileName: fileName)));
        },
        onError: (String error) {
          add(VideoFoldersCopyStatusChanged(VideoFoldersCopyStatusUnit(
              fileType: VideoFoldersFileType.folder,
              status: VideoFoldersCopyStatus.failure,
              error: error)));
        });
  }

  void _onCopyStatusChanged(
      VideoFoldersCopyStatusChanged event, Emitter<VideoFoldersState> emit) {
    emit(state.copyWith(copyStatus: event.status));
  }

  void _onCancelCopy(
      VideoFoldersCancelCopy event, Emitter<VideoFoldersState> emit) {
    _fileRepository.cancelCopy();
  }

  void _onCopyVideosSubmitted(
      VideoFoldersVideosCopySubmitted event, Emitter<VideoFoldersState> emit) {
    emit(state.copyWith(
        copyStatus: VideoFoldersCopyStatusUnit(
            fileType: VideoFoldersFileType.folder,
            status: VideoFoldersCopyStatus.start)));

    _videoRepository.copyVideosTo(
        videos: event.videos,
        dir: event.dir,
        onProgress: (fileName, current, total) {
          add(VideoFoldersCopyStatusChanged(VideoFoldersCopyStatusUnit(
              fileType: VideoFoldersFileType.video,
              status: VideoFoldersCopyStatus.copying,
              fileName: fileName,
              current: current,
              total: total)));
        },
        onDone: (fileName) {
          add(VideoFoldersCopyStatusChanged(VideoFoldersCopyStatusUnit(
              fileType: VideoFoldersFileType.video,
              status: VideoFoldersCopyStatus.success,
              fileName: fileName)));
        },
        onError: (String error) {
          add(VideoFoldersCopyStatusChanged(VideoFoldersCopyStatusUnit(
              fileType: VideoFoldersFileType.video,
              status: VideoFoldersCopyStatus.failure,
              error: error)));
        });
  }

  void _onDeleteVideosSubmitted(VideoFoldersDeleteVideosSubmitted event,
      Emitter<VideoFoldersState> emit) async {
    emit(state.copyWith(deleteStatus: VideoFoldersDeleteStatus.loading));

    try {
      await _videoRepository.deleteVideos(event.videos);

      List<VideoItem> videos = [...state.loadVideosInFolderStatus.videos];
      List<VideoItem> checkedVideos = [
        ...state.loadVideosInFolderStatus.checkedVideos
      ];

      videos.removeWhere((video) => event.videos.contains(video));
      checkedVideos.removeWhere((video) => event.videos.contains(video));

      // Recompute video count in video folders data after deleting videos.
      final videoFolders = [...state.videoFolders];
      final currentFolder = state.videoFolderOpenStatus.current;

      if (currentFolder != null) {
        final currentFolderIndex = videoFolders.indexOf(currentFolder);

        currentFolder.videoCount -= event.videos.length;
        if (currentFolder.videoCount == 0) {
          videoFolders.removeAt(currentFolderIndex);
        } else {
          videoFolders[currentFolderIndex] = currentFolder;
        }
      }

      emit(state.copyWith(
          deleteStatus: VideoFoldersDeleteStatus.success,
          videoFolders: videoFolders,
          loadVideosInFolderStatus: state.loadVideosInFolderStatus
              .copyWith(videos: videos, checkedVideos: checkedVideos)));
    } on Exception catch (e) {
      emit(state.copyWith(
          deleteStatus: VideoFoldersDeleteStatus.failure,
          failureReason: CommonUtil.convertHttpError(e)));
    }
  }

  void _onUploadVideos(
      VideoFoldersUploadVideos event, Emitter<VideoFoldersState> emit) {
    emit(state.copyWith(
        uploadStatus: VideoFoldersUploadStatusUnit(
            status: VideoFoldersUploadStatus.start)));

    _videoRepository.uploadVideos(
        videos: event.videos,
        folder: event.folder?.path,
        onError: (msg) {
          add(VideoFoldersUploadStatusChanged(
              status: VideoFoldersUploadStatusUnit(
                  status: VideoFoldersUploadStatus.failure,
                  failureReason: msg)));
        },
        onSuccess: () {
          add(VideoFoldersUploadStatusChanged(
              status: VideoFoldersUploadStatusUnit(
                  status: VideoFoldersUploadStatus.success),
              folder: event.folder,
              addedVideoCount: event.videos.length));
        },
        onUploading: (sent, total) {
          add(VideoFoldersUploadStatusChanged(
              status: VideoFoldersUploadStatusUnit(
                  status: VideoFoldersUploadStatus.uploading,
                  current: sent,
                  total: total)));
        });
  }

  void _onUploadStatusChanged(VideoFoldersUploadStatusChanged event,
      Emitter<VideoFoldersState> emit) async {
    emit(state.copyWith(uploadStatus: event.status));

    if (event.status.status == VideoFoldersUploadStatus.success) {
      final isOpened = state.videoFolderOpenStatus.isOpened;
      if (isOpened) {
        final currentFolder = state.videoFolderOpenStatus.current;
        if (currentFolder != null) {
          final videos =
              await _videoRepository.getVideosInFolder(currentFolder.id);
          emit(state.copyWith(
              loadVideosInFolderStatus:
                  state.loadVideosInFolderStatus.copyWith(videos: videos)));
        }
      } else {
        final videoFolders = await _videoRepository.getAllVideoFolders();
        emit(state.copyWith(videoFolders: videoFolders));
      }
    }
  }

  FutureOr<void> _onDownloadVideoFoldersToLocal(
      VideoFoldersDownloadVideoFoldersToLocal event,
      Emitter<VideoFoldersState> emit) async {
    emit(state.copyWith(showLoading: true));

    try {
      final bytes =
          await _videoRepository.readVideoFoldersAsBytes(event.folders);
      String fileName = "";
      if (event.folders.length == 1) {
        fileName = "${event.folders.first.path.split("/").last}.zip";
      } else if (event.folders.length > 1) {
        final timestamp = DateTime.now().millisecondsSinceEpoch;
        fileName = "videoFolders_$timestamp.zip";
      }
      CommonUtil.downloadAsWebFile(bytes: bytes, fileName: fileName);
      emit(state.copyWith(showLoading: false));
    } catch (e) {
      emit(state.copyWith(
          showLoading: false, showError: true, failureReason: e.toString()));
    }
  }

  FutureOr<void> _onDownloadVideosToLocal(
      VideoFoldersDownloadVideosToLocal event,
      Emitter<VideoFoldersState> emit) async {
    emit(state.copyWith(showLoading: true));

    try {
      final bytes = await _videoRepository.readVideosAsBytes(event.videos);
      String fileName = "";
      if (event.videos.length == 1) {
        fileName = event.videos.first.path.split("/").last;
      } else if (event.videos.length > 1) {
        final timestamp = DateTime.now().millisecondsSinceEpoch;
        fileName = "videos_$timestamp.zip";
      }
      CommonUtil.downloadAsWebFile(bytes: bytes, fileName: fileName);
      emit(state.copyWith(showLoading: false));
    } catch (e) {
      emit(state.copyWith(
          showLoading: false, showError: true, failureReason: e.toString()));
    }
  }
}
